/*****************************************************************************
**
**  cycle_blends.c
**
**  Description:
**     Contains Unigraphics entry points for the application.
**
**	Lazovich U.V.	18.03.02
**
**
**
**  Progpam look over all bodes, all features of search body, then
**  programm find edge blends with given radius and hightlight its.
**
**	For Unigraphics V17.0
**
*****************************************************************************/

/*  Include files */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <uf.h>
#include <uf_ui.h>
#include <uf_obj.h>
#include <uf_assem.h>
#include <uf_modl.h>
#include <uf_modl_types.h>
#include <uf_disp.h>
#include <uf_object_types.h>

#include "cycle_blends_03_02.h"


#define PRTERR(line)if(errorCode){PrintErrorMessage(errorCode,line);errorCode=0;goto term;}
								 

/*****************************************************************************
**  Activation Methods
*****************************************************************************/
/*  Explicit Activation
**      This entry point is used to activate the application explicitly, as in
**      "File->Execute UG/Open->User Function..." 
*/
extern DllExport void ufusr( char *parm, int *returnCode, int rlen )
{
	prterr=fopen("C:\\prterr.txt","w");
	fprintf(prterr,"cycle_blends has started	line=%d\n",__LINE__);

	errorCode=UF_initialize();PRTERR(__LINE__);


	

	{// determine work part file --------------------------------------------
		
		
		work_part=UF_ASSEM_ask_work_part();
		if(work_part==NULL_TAG)
		{
			fprintf(prterr,"there is no work part		line=%d\n",__LINE__);
			goto term;
		}	
		//fprintf(prterr,"work_part=%d	line=%d\n",work_part,__LINE__);

	}// ---------------------------------------------------------------------


back_uc1608:
	{// determine value of radius and color ---------------------------------


		
			char	*cp1=" ";
			char	cp2[][16]={"=","="};
			int		ip3=2;
			int		ia4[]={0,2};
			double	ra5[]={20.0,0.0};
			int		ip6[]={1,0};
		    
		



			errorCode=uc1608(cp1,cp2,ip3,ia4,ra5,ip6);  
    		/*fprintf(prterr,"after uc1608 = %d\n", errorCode);*/
			
			switch(errorCode)
			{
			case 1: goto back_uc1608;		/* Back */
			case 2:	goto term;				/* CANCEL */
			case 3: break;					/* OK - No user input */
			case 4: break;					/* OK - with user input */
			case 8: fprintf(prterr,"\nASK_FOR_RADIUS line=%d \n",__LINE__);
					fprintf(prterr,"errmsg=%s \n",
						"Disallowed state, unable to bring up dialog");
					goto term;
			}/*switch*/
			
			radius_ask=(float)ra5[0];
			color_ask=ia4[1];
			//fprintf(prterr,"R=%f\n",ra5[0]);
			//uc1601("determine value of radius",1);
			
	}//-----------------------------------------------------------------------


			
	{// BLEND      -------------------

			tag_t	feature=NULL_TAG;	

			//  feature
			errorCode=UF_OBJ_cycle_objs_in_part(work_part,UF_feature_type,&feature);
			PRTERR(__LINE__);
			//fprintf(prterr,"feature=%d		line=%d\n",feature,__LINE__);
			

			while(feature)//   feature
			{
				
			char*			feature_type;

			//  feature	
			errorCode=UF_MODL_ask_feat_type(feature,&feature_type);PRTERR(__LINE__);
			//fprintf(prterr,"feature_type=%s		line=%d\n",feature_type,__LINE__);
				
			if(strcmp(feature_type,"BLEND")==0)
				{//	if this feature is BLEND 
				char*		radius_blend;//must be freed
				int			edit=0;
				char*		rval="1234567890_1234567890";
				float		fval=0;
				uf_list_p_t	faces_list;
				int			cnt_faces,i;
				tag_t		obj;
				
				//  BLEND-
				errorCode=UF_MODL_ask_blend_parms(feature,edit,&radius_blend);PRTERR(__LINE__);

				//   
				rval=strchr(radius_blend,'=');
				//fprintf(prterr,"rval = %s	line=%d\n",rval,__LINE__);
				//uc1601(rval,1);

				// BLEND    flow	
				sscanf(rval+1,"%f",&fval);
				//fprintf(prterr,"fvalue=%f			line=%d\n",fval,__LINE__);

				//  
				UF_free(radius_blend);
									
				if(fabs(fval-radius_ask)<0.0001)//    
					{	

						//    
						errorCode=UF_MODL_ask_feat_faces(feature,&faces_list);PRTERR(__LINE__);
						//    
						errorCode=UF_MODL_ask_list_count(faces_list,&cnt_faces);PRTERR(__LINE__);
						for(i=0;i<cnt_faces;i++)
						{
							//   
							errorCode=UF_MODL_ask_list_item(faces_list,i,&obj);PRTERR(__LINE__);
							//    
							errorCode=UF_OBJ_set_color(obj,color_ask);PRTERR(__LINE__);
						}
						//  
						errorCode=UF_MODL_delete_list(&faces_list);PRTERR(__LINE__);

					}
				}//endif(strcmp)


			errorCode=UF_OBJ_cycle_objs_in_part(work_part,UF_feature_type,&feature);PRTERR(__LINE__);
			

			}// while(feature) 
	}//------------------------------------------------------------------------			

goto back_uc1608;//    ..

	
term://
	errorCode=UF_terminate();
	PrintErrorMessage(errorCode,__LINE__);
	fprintf(prterr,"cycle_blends has stoped		line=%d\n",__LINE__);
	fflush(prterr);
	fclose(prterr);

}




/*****************************************************************************
**  Utilities
*****************************************************************************/
/* Unload Handler
**     This function specifies when to unload your application from Unigraphics.
**     If your application registers a callback (from a MenuScript item or a
**     User Defined Object for example), this function MUST return
**     "UF_UNLOAD_UG_TERMINATE". */
extern int ufusr_ask_unload( void )
{
    return( UF_UNLOAD_IMMEDIATELY );
}
/* PrintErrorMessage
**
**     Prints error messages to standard error and the Unigraphics status
**     line. */
static void PrintErrorMessage( int errorCode,int line )
{
    if ( 0 != errorCode )
    {
        /* Retrieve the associated error message */
        char message[133];
        UF_get_fail_message( errorCode, message );

        /* Print out the message */
        UF_UI_set_status( message );

        fprintf( prterr, "%s	error_Code=%d	line=%d\n", message,errorCode,line);
	
    }
}
