/*****************************************************************************
**
**  arc_centr.c	(+ arc_centr.h)
**
**  Description:
**     Contains Unigraphics entry points for the application.
**
**	      
**	 :
**	   ,     
**   0.0  0.5  1.0.     .
**   .. ->      -> 05  2002  
*****************************************************************************/

/*  Include files */
#include <stdio.h>
#include <uf.h>
#include <uf_ui.h>
#include <uf_obj.h>
#include <uf_object_types.h>
#include <uf_disp.h>
#include <uf_modl.h>
#include <uf_vec.h>
#include <uf_curve.h>

#include "arc_centr.h"

#define PRTERR(line)if(errorCode){PrintErrorMessage(errorCode,line);errorCode=0;goto term;}

/*****************************************************************************
**  Activation Methods
*****************************************************************************/

/*  Explicit Activation
**      This entry point is used to activate the application explicitly, as in
**      "File->Execute UG/Open->User Function..." */
extern DllExport void ufusr( char *parm, int *returnCode, int rlen )
{

	prterr=fopen("C:\\prterr.txt","w");
	fprintf(prterr,"arc_centr has started	line=%d\n",__LINE__);

	errorCode=UF_initialize();PRTERR(__LINE__);

get_curve:
	{
		char	message[]="   ";
		char	title[]="   ";
		int		scope=UF_UI_SEL_SCOPE_ANY_IN_ASSEMBLY;
		int		response;
		double	cursor[3];
		tag_t	view;

		
		//    ,   
		//     prterr.txt  
		//    
		errorCode=UF_UI_select_with_single_dialog
			(message,title,scope,init_arc,NULL,&response,&object,cursor,&view);
			PRTERR(__LINE__);

		switch(response)
		{

			case UF_UI_BACK: if(object)
						{
							errorCode=UF_DISP_set_highlight(object,0);
							PRTERR(__LINE__);
							goto get_curve;
						}

			case UF_UI_CANCEL:	if(object)
						{
							errorCode=UF_DISP_set_highlight(object,0);
							PRTERR(__LINE__);
							fprintf(prterr,"user has CANCEL	line=%d\n",__LINE__);
							goto term;
						}
							goto term;

			case UF_UI_OK:	if(!object)
						{
							errorCode=UF_DISP_set_highlight(object,0);
							PRTERR(__LINE__);
							goto get_curve;
						}

			case UF_UI_OBJECT_SELECTED:
							break;

			case UF_UI_OBJECT_SELECTED_BY_NAME:
							break;
		
		}

	}


	{//---------------------------------------------------------------------

		double parm;
		double point[3];
		double tangent[3];
		double p_norm[3];
		double b_norm[3];
		double torsion;
		double rad_of_curve;

		
		//     0,0
		parm=0;
		errorCode=UF_MODL_ask_curve_props 
			(object,parm,point,tangent,p_norm,b_norm,&torsion,&rad_of_curve );
		PRTERR(__LINE__);
		//fprintf(prterr,"%6.4f		%6.4f		%6.4f \n",point[0],point[1],point[2]);
		UF_VEC3_copy(point,first_point);


		//     0,5
		parm=0.5;
		errorCode=UF_MODL_ask_curve_props 
			(object,parm,point,tangent,p_norm,b_norm,&torsion,&rad_of_curve );
		PRTERR(__LINE__);
		//fprintf(prterr,"%6.4f		%6.4f		%6.4f \n",point[0],point[1],point[2]);
		UF_VEC3_copy(point,second_point);

		
		
		//     1,0
		parm=1;
		errorCode=UF_MODL_ask_curve_props 
			(object,parm,point,tangent,p_norm,b_norm,&torsion,&rad_of_curve );
		PRTERR(__LINE__);
		//fprintf(prterr,"%6.4f		%6.4f		%6.4f \n",point[0],point[1],point[2]);
		UF_VEC3_copy(point,third_point);

	}//---------------------------------------------------------------------


	{//---------------------------------------------------------------------
		int		create_flag=1;
		tag_t	arc_tag;


		//  
		errorCode=UF_DISP_set_highlight(object,0);
		PRTERR(__LINE__);
		

		// ,      ,
		//      
		errorCode=UF_CURVE_create_arc_thru_3pts 
			(create_flag,first_point,second_point,third_point,&arc_tag );
		//fprintf(prterr,"errorCode=%d\n",errorCode);
		PRTERR(__LINE__);
	}//---------------------------------------------------------------------
	
	
	//    
	goto get_curve; 



// 
term:
	errorCode=UF_terminate();
	if(errorCode)
		PrintErrorMessage(errorCode,__LINE__);
	fprintf(prterr,"arc_centr has completed	line=%d\n",__LINE__);
	fflush(prterr);
	fclose(prterr);


}
/*****************************************************************************
**  Utilities
*****************************************************************************/

/* Unload Handler
**     This function specifies when to unload your application from Unigraphics.
**     If your application registers a callback (from a MenuScript item or a
**     User Defined Object for example), this function MUST return
**     "UF_UNLOAD_UG_TERMINATE". */
static int ufusr_ask_unload( void )
{
    return( UF_UNLOAD_IMMEDIATELY );
}

/* PrintErrorMessage
**
**     Prints error messages to standard error and the Unigraphics status
**     line. */
static void PrintErrorMessage( int errorCode, int line )
{
    if ( 0 != errorCode )
    {
        /* Retrieve the associated error message */
        char message[133];
        UF_get_fail_message( errorCode, message );

        /* Print out the message */
        UF_UI_set_status( message );

        fprintf(prterr, "%s		line=%d\n", message,line);
    }
}

static int init_arc(UF_UI_selection_p_t select, void* user_data)
{
	int num_triples=5;
		UF_UI_mask_t	mask_triples[]={
		UF_line_type,0,0,
		UF_circle_type,0,0,
		UF_conic_type,0,0,
		UF_spline_type,0,0,
		UF_solid_type,UF_solid_edge_subtype,1
	};

	errorCode=UF_UI_set_sel_mask(select, UF_UI_SEL_MASK_CLEAR_AND_ENABLE_SPECIFIC,
								num_triples, mask_triples);
	if (errorCode==0)
		return UF_UI_SEL_SUCCESS;
	else
		return UF_UI_SEL_FAILURE;
}
