/*****************************************************************************
**
**  list_components.c
**
**  Description:
**     Contains Unigraphics entry points for the application.
**
*****************************************************************************/

/*  Include files */
#include <stdio.h>
#include <uf.h>
#include <uf_ui.h>
#include "list_components.h"

/*****************************************************************************
**  Activation Methods
*****************************************************************************/

/*  Explicit Activation
**      This entry point is used to activate the application explicitly, as in
**      "File->Execute UG/Open->User Function..." */
extern DllExport void ufusr( char *parm, int *returnCode, int rlen )
{
    /* Initialize the API environment */
    int errorCode = UF_initialize();

	prterr=fopen("C:\\prterr.txt","w");

    if ( 0 == errorCode )
    {
        /* TODO: Add your application code here */

		list_components();

        /* Terminate the API environment */
        errorCode = UF_terminate();
    }

    /* Print out any error messages */
    PrintErrorMessage( errorCode,__LINE__ );
	fflush(prterr);
	fclose(prterr);
}
/*****************************************************************************
**  Utilities
*****************************************************************************/

/* Unload Handler
**     This function specifies when to unload your application from Unigraphics.
**     If your application registers a callback (from a MenuScript item or a
**     User Defined Object for example), this function MUST return
**     "UF_UNLOAD_UG_TERMINATE". */
extern int ufusr_ask_unload( void )
{
    return( UF_UNLOAD_IMMEDIATELY );
}

/* PrintErrorMessage
**
**     Prints error messages to standard error and the Unigraphics status
**     line. */
static void PrintErrorMessage( int errorCode, int line )
{
    if ( 0 != errorCode )
    {
        /* Retrieve the associated error message */
        char message[133];
        UF_get_fail_message( errorCode, message );

        /* Print out the message */
        UF_UI_set_status( message );

        fprintf( prterr, "%s     line=%d\n", message,line );
    }
}

//**********************************************************************************
//**********************************************************************************
//**********************************************************************************
//**********************************************************************************
#include <stdio.h>
#include <uf.h>
#include <uf_ui.h>
#include <uf_object_types.h>
#include <uf_disp.h>
#include <uf_assem.h>
#include <uf_ugmgr.h>
#include <string.h>
#include <uf_drf.h>
#include <uf_vec.h>
#include <uf_csys.h>


static int init_proc(UF_UI_selection_p_t select, void* user_data)
{

	int	num_triples=1;
	UF_UI_mask_t	mask_triples[]={UF_component_type,
		UF_component_subtype,0};

	irc=UF_UI_set_sel_mask(select,UF_UI_SEL_MASK_CLEAR_AND_ENABLE_SPECIFIC,
		num_triples,mask_triples);
	if(irc==0)
	{
		return (UF_UI_SEL_SUCCESS);
	}
	else
	{
		return (UF_UI_SEL_FAILURE);
	}

}
static void list_components(void)
{
start_loop:
	{//select with class dialog --------------------------------------------------
		char	cue[]="Select components";
		char	title[]="Select component";
		int		response,count=0,i;
		


		//0 - select objects on Any View
		irc=UF_UI_set_cursor_view(0);
		if(irc)
		{
			PrintErrorMessage(irc,__LINE__);
			goto term;
		}		
		
		
back_select_with_class_dialog:
		irc=UF_UI_select_with_class_dialog(cue,title,
			UF_UI_SEL_SCOPE_NO_CHANGE,init_proc,NULL,&response,&count,&objects);
		if(irc)
		{
			PrintErrorMessage(irc,__LINE__);
			goto term;
		}
		
		switch(response)
			{
			case	UF_UI_BACK:
				if(count)
					UF_free(objects);
				goto back_select_with_class_dialog;

			case	UF_UI_CANCEL:
				if(count)
					UF_free(objects);
				fprintf(prterr,"Operator selected CANCEL  line=%d\n",__LINE__);
				goto term;
			case	UF_UI_OK:		;
				if(count)
				{
					fprintf(prterr,"count_objects=%d      line=%d\n",count,__LINE__);
					break;
				}
				else
					goto back_select_with_class_dialog;
			}
			
			for(i=0;i<count;i++)
			{
				irc=UF_DISP_set_highlight(objects[i],0);
				if(irc)
				{
					PrintErrorMessage(irc,__LINE__);
					goto term;
				}
			}
			
			object_count=count;

	}//select with class dialog --------------------------------------------------

//********************************************************************************
//********************************************************************************

	{//for for for for for for for for for for for for for for for for -----------
		int	i;
	for(i=0;i<object_count;i++)
	{
	
	{//ask component data --------------------------------------------------------

		
	
		char	refset_name[30+1];
		char	instance_name[30+1];
		double	origin[3];
		double	csys_matrix[9];
		double	transform[4][4];

		irc=UF_ASSEM_ask_component_data(objects[i],part_name,refset_name,
			instance_name,origin,csys_matrix,transform);
		if(irc)
		{
			PrintErrorMessage(irc,__LINE__);
			goto term;
		}
		
		fprintf(prterr,"part_name=%s       line=%d\n",part_name,__LINE__);


	}//ask component data --------------------------------------------------------

	{//decode part filename ------------------------------------------------------

	
		
		char	part_revision[UF_UGMGR_PARTREV_SIZE+1];
		char	part_file_type[UF_UGMGR_FTYPE_SIZE+1];
		char	part_file_name[UF_UGMGR_FNAME_SIZE+1];


		irc=UF_UGMGR_decode_part_filename(part_name,part_number,part_revision,
			part_file_type,part_file_name);
		if(irc)
		{
			PrintErrorMessage(irc,__LINE__);
			goto term;
		}

		/*
		fprintf(prterr,"part_number=%s   line=%d\n",part_number,__LINE__);
		fprintf(prterr,"part_revision=%s   line=%d\n",part_revision,__LINE__);
		fprintf(prterr,"part_file_type=%s   line=%d\n",part_file_type,__LINE__);
		fprintf(prterr,"part_file_name=%s   line=%d\n",part_file_name,__LINE__);
		*/

	}//decode part filename ------------------------------------------------------
	strcpy(text_string[i],part_number);
	}
	}//for for for for for for for for for for for for for for -------------------



//*********************************************************************************
//*********************************************************************************

	
	{//screen position point -----------------------------------------------------

		char*	message1="Select point leader";
		double	point1[3],point2[3];
		char*	message2="Select message point";

		
		//0 - select objects on Work View
		irc=UF_UI_set_cursor_view(1);
		if(irc)
		{
			PrintErrorMessage(irc,__LINE__);
			goto term;
		}

back_point1:
		irc=uc1615(message1,point1);
		switch(irc)
		{
		case	1:	
			goto back_point1;
		case	2:	
			fprintf(prterr,"selected CANCEL  line=%d\n",__LINE__);
			goto	term;
		case	5:
			break;
		case	7:
			fprintf(prterr,"no Active Part  line=%d\n",__LINE__);
			goto	term;
		case	8:
			fprintf(prterr,"Dissalowed state, unable to bring up dialog		line=%d\n",__LINE__);
			goto	term;
		}
	
		/*
		fprintf(prterr,"point1	[0]=%6.4f	[1]=%6.4f	[1]=%6.4f	\n",
			point1[0],point1[1],point1[2]);
		*/
		UF_VEC3_copy(point1,model_pos_3d);






back_point2:
		irc=uc1615(message2,point2);
		switch(irc)
		{
		case	1:	
			goto back_point2;
		case	2:	
			fprintf(prterr,"selected CANCEL  line=%d\n",__LINE__);
			goto	term;
		case	5:
			break;
		case	7:
			fprintf(prterr,"no Active Part  line=%d\n",__LINE__);
			goto	term;
		case	8:
			fprintf(prterr,"Dissalowed state, unable to bring up dialog		line=%d\n",__LINE__);
			goto	term;
		}
		/*
		fprintf(prterr,"point2	[0]=%6.4f	[1]=%6.4f	[1]=%6.4f	\n",
			point2[0],point2[1],point2[2]);
		*/
		UF_VEC3_copy(point2,origin_3d);



	
	}//screen position point -----------------------------------------------------
	
	{//create label --------------------------------------------------------------


		
		
		UF_DRF_leader_attach_type_t		
			leader_attach_type=UF_DRF_leader_attach_screen;
		UF_DRF_object_p_t	object=NULL;
		
		tag_t	label_tag;
		
	

	

		irc=UF_DRF_create_label(object_count,text_string,origin_3d,
			leader_attach_type,object,model_pos_3d,&label_tag);
		if(irc)
		{
			PrintErrorMessage(irc,__LINE__);
			goto term;
		}



	}//create label --------------------------------------------------------------

	goto start_loop;
term:
	return;
}
